<?php

namespace Drupal\invite\Controller;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\invite\InviteAcceptEvent;
use Drupal\invite\InviteConstants;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcher;

/**
 * Class to Invite Accept.
 *
 * @package Drupal\invite\Controller
 */
class InviteAccept extends ControllerBase {

  /**
   * The event dispatcher.
   *
   * @var \Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher
   */
  public $dispatcher;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Config object.
   *
   * @var config
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('event_dispatcher'),
      $container->get('messenger'),
      $container->get('config.factory')
    );
  }

  /**
   * Construct InviteAccept object.
   */
  public function __construct(EventDispatcher $dispatcher, MessengerInterface $messenger, ConfigFactory $config) {
    $this->dispatcher = $dispatcher;
    $this->messenger = $messenger;
    $this->config = $config->get('invite.invite_config');
  }

  /**
   * Accepts an invitation.
   */
  public function accept($invite) {
    $account = $this->currentUser();
    $redirect = '<front>';
    $message = 'Hmm.';
    $type = 'status';

    // Current user is the inviter.
    if ($account->id() == $invite->getOwnerId()) {
      $message = $this->t("You can't use your own invite.");
      $type = 'error';
    }
    // Invite has already been used.
    elseif ($invite->getStatus() == InviteConstants::INVITE_USED) {
      $message = $this->t('Sorry, this invitation has already been used.');
      $type = 'error';
    }

    // Invite has already been withdrawn.
    elseif ($invite->getStatus() == InviteConstants::INVITE_WITHDRAWN) {
      $message = $this->t('Sorry, this invitation has already been withdrawn.');
      $type = 'error';
    }

    // Invite is expired.
    elseif ($invite->expires->value < time()) {
      $message = $this->t('Sorry, this invitation is expired.');
      $type = 'error';
      $invite->setStatus(InviteConstants::INVITE_EXPIRED);
      $invite->save();
    }

    // Good to go!
    else {
      $_SESSION['invite_code'] = $invite->getRegCode();
      $message = $this->t('Please create an account to accept the invitation.');

      // Redirect invitee to configured route when accepting the invite.
      $accept_redirect = $this->config('invite.invite_config')->get('accept_redirect');

      if ($accept_redirect) {
        $redirect = $accept_redirect;
      }
      else {
        $redirect = 'user.register';
      }
    }

    // Let other modules act on the invite accepting before the user is created.
    $invite_accept = new InviteAcceptEvent([
      'redirect' => &$redirect,
      'message' => &$message,
      'type' => &$type,
      'invite' => &$invite,
    ]);

    $this->dispatcher->dispatch($invite_accept, 'invite_accept');

    if ($type == 'error') {
      $this->messenger->addError($message);
    }
    else {
      $this->messenger->addStatus($message);
    }

    return $this->redirect($redirect);
  }

}
