<?php

namespace Drupal\invite\Form;

use Drupal\Core\Block\BlockManager;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\invite\Entity\InviteSender;
use Drupal\invite\InvitePluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for Invite type edit forms.
 *
 * @package Drupal\invite\Form
 */
class InviteTypeForm extends EntityForm {

  use StringTranslationTrait;
  /**
   * Plugin Manager.
   *
   * @var \Drupal\invite\InvitePluginManager
   */
  public $pluginManager;

  /**
   * Block Manager.
   *
   * @var \Drupal\core\Block\BlockManager
   */
  public $blockManager;

  /**
   * Database.
   *
   * @var \Drupal\core\Database\connection
   */
  protected $database;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The invite type.
   *
   * @var \Drupal\invite\Entity\InviteType
   */
  protected $inviteType;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.invite'),
      $container->get('plugin.manager.block'),
      $container->get('database'),
      $container->get('messenger'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(InvitePluginManager $plugin_manager, BlockManager $block_plugin_manager, Connection $database, MessengerInterface $messenger, ConfigFactoryInterface $config_factory) {
    $this->pluginManager = $plugin_manager;
    $this->database = $database;
    $this->blockManager = $block_plugin_manager;
    $this->messenger = $messenger;
    $this->configFactory = $config_factory;
  }

  /**
   * Helper function to load the default send method for the invite type.
   */
  public function getDefaultSendMethods($invite_type) {
    $defaults = [];
    $config = $this->configFactory()->get('invite.invite_sender.' . $invite_type->getType());
    $sending_methods = $config->get('sending_methods');
    foreach (explode('||', $sending_methods) as $sending_method) {
      if ($sending_method != '0') {
        $defaults[$sending_method] = $sending_method;
      }
    }

    return $defaults;
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    $this->inviteType = $this->entity;

    /** @var \Drupal\invite\Entity\InviteType $inviteType */
    $inviteType = $this->inviteType;
    $is_new = $inviteType->isNew();
    if ($is_new) {
      $inviteType
        ->set('label', '')
        ->set('type', '')
        ->set('description', '')
        ->setData('');

    }
    $data = unserialize($inviteType->getData());

    $form['label'] = [
      '#title' => $this->t('Invite Type Label'),
      '#type' => 'textfield',
      '#default_value' => $inviteType->label(),
      '#description' => $this->t('The human-readable name of this invite type. This name must be unique.'),
      '#required' => TRUE,
      '#size' => 30,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $inviteType->getType(),
      '#maxlength' => 255,
      '#disabled' => !$is_new,
      '#machine_name' => [
        'exists' => ['Drupal\invite\Entity\InviteType', 'load'],
        'source' => ['label'],
      ],
      '#description' => $this->t('A unique machine-readable name for this invite type. It must only contain lowercase letters, numbers, and underscores.'),
    ];

    $form['description'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#description' => $this->t('Description about the invite type.'),
      '#rows' => 5,
      '#default_value' => $inviteType->getDescription(),
    ];

    $options[] = '- ' . $this->t('None') . ' -';
    $roles = $this->entityTypeManager->getStorage('user_role')->loadMultiple();

    /** @var \Drupal\user\Entity\Role $user_role */
    foreach ($roles as $user_role) {
      if (empty($user_role->get('_core'))) {
        $options[$user_role->id()] = $user_role->label();
      }
    }
    $form['target_role'] = [
      '#type' => 'select',
      '#required' => FALSE,
      '#title' => $this->t('Role'),
      '#description' => $this->t('Please select a role to apply to the invitee (Optional).'),
      '#options' => $options,
      '#default_value' => $data['target_role'] ?? '',
    ];

    // List the available sending methods.
    $plugin_definitions = $this->pluginManager->getDefinitions();
    if (!empty($plugin_definitions)) {
      $options = [];
      foreach ($plugin_definitions as $plugin_definition) {
        $options[$plugin_definition['provider']] = $plugin_definition['id'];
      }
      $default_send_method = [];
      if (!$is_new) {
        $default_send_method = $this->getDefaultSendMethods($inviteType);
      }
      $form['send_method'] = [
        '#type' => 'checkboxes',
        '#required' => TRUE,
        '#title' => $this->t('Sending Method'),
        '#default_value' => $default_send_method,
        '#options' => $options,
      ];
    }
    else {
      $this->messenger->addWarning($this->t('Please enable a sending method module such as Invite by email.'));
      $form['send_method'] = [
        '#type' => 'item',
        '#markup' => $this->t('Please enable a sending method module such as Invite by email.'),
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    $actions = parent::actions($form, $form_state);
    if (empty($this->pluginManager->getDefinitions())) {
      $actions['submit']['#disabled'] = TRUE;
    }
    return $actions;
  }

  /**
   * Helper method to add an invite_sender record.
   */
  public function updateInviteSender($send_methods, $invite_type) {
    $type = $invite_type->getType();
    $send_methods = implode('||', $send_methods);
    $invite_sender = InviteSender::load($type);
    if (empty($invite_sender)) {
      $invite_sender = InviteSender::create(
        [
          'id' => $type,
          'sending_methods' => $send_methods,
        ]
      );
    }
    else {
      $invite_sender
        ->set('id', $type)
        ->set('sending_methods', $send_methods);
    }
    $invite_sender->save();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Assemble data.
    $data_value = $form_state->getValue('data');
    $data = !empty($data_value) ? $data_value : [];
    $data['target_role'] = $form_state->getValue('target_role');
    $form_state->setValue('data', serialize($data));
    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $inviteType = $this->entity;
    // Add/update sending invite_sender.
    try {
      $this->updateInviteSender($form_state->getValue('send_method'), $inviteType);
    }
    catch (\Exception $e) {
      throw $e;
    }

    $status = $inviteType->save();

    switch ($status) {
      case SAVED_NEW:
        $this->messenger->addStatus($this->t('Created the %label Invite type.', [
          '%label' => $inviteType->label(),
        ]));
        break;

      default:
        $this->messenger->addStatus($this->t('Saved the %label Invite type.', [
          '%label' => $inviteType->label(),
        ]));
    }
    // Reload blocks.
    $this->blockManager->clearCachedDefinitions();

    $form_state->setRedirect('entity.invite_type.collection', ['invite_type' => $inviteType->id()]);
  }

}
